from mpi4py import MPI
from ngsolve import *
from math import pi;

def mesh():
    import netgen.geom2d as geom2d;
    geo = geom2d.SplineGeometry()
    p1 = geo.AppendPoint (0,0)
    p2 = geo.AppendPoint (1,0)
    p3 = geo.AppendPoint (1,1)
    p4 = geo.AppendPoint (0,1)

    geo.Append (["line", p1, p2], bc=1,hpref=1)
    geo.Append (["line", p2, p3], bc=2,hpref=1)
    geo.Append (["line", p3, p4], bc=1,hpref=1)
    geo.Append (["line", p4, p1], bc=2,hpref=1)

    mesh = geo.GenerateMesh (maxh=0.125,quad_dominated=True)

    return Mesh(mesh)


uex=sin(2*pi*x)*sin(4*pi*y);
u_ex2=pow(2*pi,2)*uex+pow(4*pi,2)*uex;


epsilon=1E-03
sigma=0.125;
e=[];

for m in range(0,2):
    p=10;
    print("p=",p);

    meshX=mesh();
    meshX.RefineHP(m,sigma)

    Vx=H1(meshX,order=p,dirichlet=[1,2]);

    blfA=BilinearForm(Vx);
    blfA+=Laplace(epsilon);
    blfA+=Mass(1.0);

    blfA.Assemble();

    f=LinearForm(Vx);
    f+=Source(epsilon*u_ex2 + uex);

    f.Assemble();


    u=GridFunction(Vx);

    u.vec.data=blfA.mat.Inverse(Vx.FreeDofs(),inverse='pardisospd')*f.vec;



    e.append(sqrt(Integrate((uex-u)*(uex-u),meshX,order=2*p+1)));

    print(e);

Draw(u)
